<?php

namespace elanpl\L3;

class ViewEngineInfo{
    public $extension;
    public $class;
    public $config;

    public function __construct($extension, $class, $config)
    {
        $this->extension = $extension;
        $this->class = $class;
        $this->config = $config;
    }
}